#include "mainwindow.h"
#include "ui_mainwindow.h"

#include "qfunc3df.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    //     
    ui->mainToolBar->addAction(ui->actionRestore);

    //  
    grLay = new QGridLayout(ui->frame);
    //  
    grLay->setContentsMargins(1,1,1,1);

    //  
    surf = new Qwt3D::SurfacePlot(ui->frame);
    //   
    surf->setTitle("SurfacePlot from Array");

    //     
    grLay->addWidget(surf,0,0);

    //    
    QFunc3DF func(surf);
    //     (X,Y)
    func.setDomain(-1.5,1.5,-1.5,1.5);
    //     
    func.setMesh(31,21);
    //    
    func.create();

    //    
    surf->setCoordinateStyle(Qwt3D::FRAME); // Qwt3D::BOX
//    surf->setPlotStyle(Qwt3D::FILLED);
//    surf->setOrtho(false);

    //   
    QStringList labs;
    labs << "X" << "Y" << "Z";
    int num[12] = {0,1,2,0,0,0,1,1,1,2,2,2};
    for (int k=0; k != (int)surf->coordinates()->axes.size(); k++)
    {
        //  
        surf->coordinates()->axes[k].setLabelString(labs[num[k]]);
        //  
        surf->coordinates()->axes[k].setMajors(5);  // 
        surf->coordinates()->axes[k].setMinors(5);  // 
    }
    //  
    surf->coordinates()->adjustNumbers(6);  //    
    surf->coordinates()->adjustLabels(12);  //    

    //     
    rotateLab = new QLabel(ui->statusBar);  //  
    ui->statusBar->addWidget(rotateLab,0);
    shiftLab = new QLabel(ui->statusBar);   // 
    ui->statusBar->addWidget(shiftLab,0);
    scaleLab = new QLabel(ui->statusBar);   //   
    ui->statusBar->addWidget(scaleLab,0);
    zoomLab = new QLabel(ui->statusBar);   //  
    ui->statusBar->addWidget(zoomLab,0);

    //    
        //  
    connect(surf, SIGNAL(rotationChanged(double,double,double)), this, SLOT(procRotate(double,double,double)));
    connect(surf, SIGNAL(rotationChanged(double,double,double)), this, SLOT(showRotate(double,double,double)));
        //  
    connect(surf, SIGNAL(vieportShiftChanged(double,double)), this, SLOT(showShift(double,double)));
        //     
    connect(surf, SIGNAL(scaleChanged(double,double,double)), this, SLOT(showScale(double,double,double)));
        //   
    connect(surf, SIGNAL(zoomChanged(double)), this, SLOT(showZoom(double)));

    //    
    on_actionRestore_triggered();
    showShift(0,0);
}

MainWindow::~MainWindow()
{
    // 
    delete surf;    // 
    delete grLay;   // 

    delete ui;
}

void MainWindow::on_actionRestore_triggered()
{
    //   
    surf->setRotation(32,0,15);     //  
    surf->setViewportShift(0,0);    // 
    surf->setScale(0.8,0.9,2);      //   
    surf->setZoom(0.9);             //  

    surf->updateData();
    surf->updateGL();
}

//     
void MainWindow::procRotate(double x,double y,double z)
{
    x *= M_PI/180;
    y *= M_PI/180;
    z *= M_PI/180;

    int gl = Qwt3D::NOSIDEGRID;

    double sp = cos(x)*sin(z) + sin(x)*sin(y)*cos(z);
    if (sp < 0) gl |= Qwt3D::LEFT;
    if (sp > 0) gl |= Qwt3D::RIGHT;

    sp = cos(x)*cos(z) - sin(x)*sin(y)*sin(z);
    if (sp < 0) gl |= Qwt3D::FRONT;
    if (sp > 0) gl |= Qwt3D::BACK;

    sp = -sin(x)*cos(y);
    if (sp < 0) gl |= Qwt3D::FLOOR;
    if (sp > 0) gl |= Qwt3D::CEIL;

    surf->coordinates()->setGridLines(true,false,gl);
}

void MainWindow::showRotate(double x,double y,double z) {
    rotateLab->setText(
        " Angles (" +
            QString::number(x,'g',3) + ", " +
            QString::number(y,'g',3) + ", " +
            QString::number(z,'g',3) + ")  ");
}

void MainWindow::showShift(double x,double y) {
    shiftLab->setText(
        " Shifts (" +
            QString::number(x,'g',3) + ", " +
            QString::number(y,'g',3) + ")  ");
}

void MainWindow::showScale(double x,double y,double z) {
    scaleLab->setText(
        " Scales (" +
            QString::number(x,'g',3) + ", " +
            QString::number(y,'g',3) + ", " +
            QString::number(z,'g',3) + ")  ");
}

void MainWindow::showZoom(double z) {
    zoomLab->setText(
        " Zoom "  +
            QString::number(z,'g',3) + "  ");
}
